@@ -14,7 +14,7 @@
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 
 #define FAKE_ICON_SIZE 4
 
@@ -138,6 +138,8 @@ typedef struct _PolygonObject
 
     void *effectParameters;             /* Pointer to a struct that can contain
 					   custom parameters for an individual effect */
+
+    float boundSphereRadius;            // Radius of bounding sphere
 } PolygonObject;
 
 typedef struct _Clip4Polygons	// Rectangular clips
@@ -184,7 +186,7 @@ typedef struct _PolygonSet	// Polygon objects with same thickness
 
     Bool includeShadows;        // include shadows in polygon
 
-    void (*extraPolygonTransformationFunc) (PolygonObject *);
+    void (*extraPolygonTransformFunc) (PolygonObject *);
 } PolygonSet;
 
 typedef struct _WaveParam
@@ -211,8 +213,6 @@ typedef struct _Object
 {
     Point gridPosition;		// position on window in [0,1] range
     Point position;		// position on screen
-    Point3d posRel3d;		/* position relative to model center
-				   (for 3d looking effects) */
 
     // Texture x, y coordinates will be offset by given amounts
     // for quads that fall after and before this object in x and y directions.
@@ -233,11 +233,7 @@ typedef struct _Model
 
     Vector scale;
     Point scaleOrigin;
-    Point topLeft;
-    Point bottomRight;
 
-    int magicLampWaveCount;
-    WaveParam *magicLampWaves;
     WindowEvent forWindowEvent;
     float topHeight;
     float bottomHeight;
@@ -405,10 +401,10 @@ typedef struct _AnimDisplay
 {
     int screenPrivateIndex;
     Atom winIconGeometryAtom;
-    Atom wmNameAtom;
     HandleEventProc handleEvent;
     HandleCompizEventProc handleCompizEvent;
     int activeWindow;
+    CompMatch neverAnimateMatch;
 } AnimDisplay;
 
 typedef struct _PluginEventInfo
@@ -533,6 +529,7 @@ typedef struct _AnimScreen
     DamageWindowRectProc damageWindowRect;
     AddWindowGeometryProc addWindowGeometry;
     DrawWindowTextureProc drawWindowTexture;
+    InitWindowWalkerProc initWindowWalker;
 
     WindowResizeNotifyProc windowResizeNotify;
     WindowMoveNotifyProc windowMoveNotify;
@@ -545,9 +542,6 @@ typedef struct _AnimScreen
 
     Bool pluginActive[NUM_WATCHED_PLUGINS];
 
-    Bool switcherWinOpeningSuppressed; /* whether switcher window opening
-					  animation is suppressed yet */
-
     Window *lastClientListStacking; // to store last known stacking order
     int nLastClientListStacking;
     int markAllWinCreatedCountdown;
@@ -565,6 +559,8 @@ typedef struct _AnimScreen
     unsigned int nMinimizeRandomEffects;
     unsigned int nShadeRandomEffects;
 
+    CompOutput *output;
+
     OptionSets *eventOptionSets[NUM_EVENTS];
 } AnimScreen;
 
@@ -576,8 +572,6 @@ typedef struct _AnimWindow
     unsigned int state;
     unsigned int newState;
 
-    char *wmName;
-
     PolygonSet *polygonSet;
     float mvm[16];
 
@@ -587,9 +581,6 @@ typedef struct _AnimWindow
     XRectangle icon;
     XRectangle origWindowRect;
 
-    XRectangle lastKnownCoords;	/* used to determine if paintWindow is drawing
-				   on the viewport that the animation started */
-
     float numZoomRotations;
     GLushort storedOpacity;
     float timestep;		// to be used in updateWindowAttribFunc
@@ -626,9 +617,17 @@ typedef struct _AnimWindow
     Bool deceleratingMotion;	// For effects that have decel. motion
 
     int curAnimSelectionRow;
+    int prevAnimSelectionRow;	// For the case when one event interrupts another
+
+    CompTransform transform;
+
+    Box BB;       // Bounding box for damage region calc. of CompTransform fx
+    Box lastBB;   // Last bounding box
 
     // for magic lamp
     Bool minimizeToTop;
+    int magicLampWaveCount;
+    WaveParam *magicLampWaves;
 
     // for glide effect
     float glideModRotAngle;	// The angle of rotation modulo 360
@@ -654,11 +653,9 @@ typedef struct _AnimWindow
     CompWindow *dodgeChainPrev;	// for dodging windows
     CompWindow *dodgeChainNext;	// for dodging windows
     Bool skipPostPrepareScreen;
-    Bool drawnOnHostSkip;
- 
-    // for airplane
-    float airplanePathLength;
-    Bool airplaneFly2TaskBar;
+    Bool walkerOverNewCopy;     // whether walker is on the copy at the new pos.
+    unsigned int walkerVisitCount; // how many times walker has visited this window
+    CompWindow *winPassingThrough; // win. passing through this one during focus effect
 } AnimWindow;
 
 typedef struct _AnimEffectProperties
@@ -667,7 +664,7 @@ typedef struct _AnimEffectProperties
 				    WindowPaintAttrib *);
     void (*prePaintWindowFunc) (CompScreen *, CompWindow *);
     void (*postPaintWindowFunc) (CompScreen *, CompWindow *);
-    Bool (*animStepFunc) (CompScreen *, CompWindow *, float time);
+    void (*animStepFunc) (CompScreen *, CompWindow *, float time);
     void (*initFunc) (CompScreen *, CompWindow *);
     void (*initGridFunc) (AnimScreen *, AnimWindow *, int *, int *);
     void (*addCustomGeometryFunc) (CompScreen *, CompWindow *, int, Box *,
@@ -679,28 +676,30 @@ typedef struct _AnimEffectProperties
     void (*updateWindowTransformFunc)
     (CompScreen *, CompWindow *, CompTransform *);
     void (*postPreparePaintScreenFunc) (CompScreen *, CompWindow *);
+    void (*updateBBFunc) (CompOutput *, CompWindow *);
 } AnimEffectProperties;
 
 AnimEffectProperties *animEffectPropertiesTmp;
 
 #define GET_ANIM_DISPLAY(d)						\
-    ((AnimDisplay *) (d)->privates[animDisplayPrivateIndex].ptr)
+    ((AnimDisplay *) (d)->base.privates[animDisplayPrivateIndex].ptr)
 
 #define ANIM_DISPLAY(d)				\
     AnimDisplay *ad = GET_ANIM_DISPLAY (d)
 
 #define GET_ANIM_SCREEN(s, ad)						\
-    ((AnimScreen *) (s)->privates[(ad)->screenPrivateIndex].ptr)
+    ((AnimScreen *) (s)->base.privates[(ad)->screenPrivateIndex].ptr)
 
 #define ANIM_SCREEN(s)							\
     AnimScreen *as = GET_ANIM_SCREEN (s, GET_ANIM_DISPLAY (s->display))
 
 #define GET_ANIM_WINDOW(w, as)						\
-    ((AnimWindow *) (w)->privates[(as)->windowPrivateIndex].ptr)
+    ((AnimWindow *) (w)->base.privates[(as)->windowPrivateIndex].ptr)
 
-#define ANIM_WINDOW(w)							\
-    AnimWindow *aw =                                                    \
-	GET_ANIM_WINDOW (w, GET_ANIM_SCREEN (w->screen,	GET_ANIM_DISPLAY (w->screen->display)))
+#define ANIM_WINDOW(w)					     \
+    AnimWindow *aw = GET_ANIM_WINDOW (w,                     \
+		     GET_ANIM_SCREEN (w->screen,             \
+		     GET_ANIM_DISPLAY (w->screen->display)))
 
 #define NUM_OPTIONS(s) (sizeof ((s)->opt) / sizeof (CompOption))
 
@@ -723,8 +722,20 @@ AnimEffectProperties *animEffectPropertiesTmp;
 
 // ratio of perceived length of animation compared to real duration
 // to make it appear to have the same speed with other animation effects
+
+#define DOMINO_PERCEIVED_T 0.8f
+#define DREAM_PERCEIVED_T 0.6f
+#define EXPLODE_PERCEIVED_T 0.7f
+#define FOLD_PERCEIVED_T 0.55f
+#define LEAFSPREAD_PERCEIVED_T 0.6f
+#define ROLLUP_PERCEIVED_T 0.6f
+#define SKEWER_PERCEIVED_T 0.6f
+
+#define SPRINGY_ZOOM_PERCEIVED_T 0.55f
+#define NONSPRINGY_ZOOM_PERCEIVED_T 0.6f
 #define ZOOM_PERCEIVED_T 0.75f
 
+
 /*
  * Function prototypes
  *
@@ -736,14 +747,11 @@ void
 modelInitObjects (Model * model,
 		  int x, int y,
 		  int width, int height);
- 
+
 void
 postAnimationCleanup (CompWindow * w,
 		      Bool resetAnimation);
- 
-void
-modelCalcBounds (Model * model);
- 
+
 float
 defaultAnimProgress (AnimWindow * aw);
 
@@ -753,23 +761,24 @@ sigmoidAnimProgress(AnimWindow * aw);
 float
 decelerateProgressCustom (float progress,
 			  float minx, float maxx);
- 
+
 float
 decelerateProgress (float progress);
- 
+
 void
 applyTransformToObject (Object *obj, GLfloat *mat);
- 
-Bool polygonsAnimStep (CompScreen * s,
-		       CompWindow * w,
-		       float time);
+
+void
+polygonsAnimStep (CompScreen *s,
+		  CompWindow *w,
+		  float time);
 
 AnimDirection 
 getAnimationDirection (CompWindow * w,
 		       CompOptionValue *value,
 		       Bool openDir);
 
-Bool
+void
 defaultAnimStep (CompScreen * s,
 		 CompWindow * w,
 		 float time);
@@ -788,12 +797,38 @@ defaultMinimizeUpdateWindowTransform(CompScreen *s,
 				     CompWindow *w,
 				     CompTransform *wTransform);
 
+Bool
+animZoomToIcon (AnimScreen *as, AnimWindow *aw);
+
 void
 animDrawWindowGeometry(CompWindow * w);
 
 Bool
 getMousePointerXY(CompScreen * s, short *x, short *y);
 
+void
+expandBoxWithBox (Box *target, Box *source);
+
+void
+expandBoxWithPoint (Box *target, float fx, float fy);
+
+void
+updateBBWindow (CompOutput *output,
+		CompWindow * w);
+
+void
+updateBBScreen (CompOutput *output,
+		CompWindow * w);
+
+void
+compTransformUpdateBB (CompOutput *output,
+		       CompWindow *w);
+
+void
+prepareTransform (CompScreen *s,
+		  CompOutput *output,
+		  CompTransform *resultTransform,
+		  CompTransform *transform);
 
 /* airplane3d.c */
 
@@ -801,7 +836,7 @@ void
 fxAirplane3DInit (CompScreen *s,
 		  CompWindow *w);
 
-Bool
+void
 fxAirplane3DAnimStep (CompScreen * s,
 		      CompWindow * w,
 		      float time);
@@ -816,7 +851,7 @@ fxAirplane3DDrawCustomGeometry (CompScreen * s,
 				CompWindow * w);
 
 void 
-AirplaneExtraPolygonTransformationFunc (PolygonObject * p);
+AirplaneExtraPolygonTransformFunc (PolygonObject * p);
 
 
 /* beamup.c */
@@ -826,7 +861,7 @@ fxBeamupUpdateWindowAttrib (AnimScreen *as,
 			    CompWindow * w,
 			    WindowPaintAttrib *wAttrib);
 
-Bool
+void
 fxBeamUpModelStep (CompScreen *s,
 		   CompWindow *w,
 		   float time);
@@ -837,7 +872,7 @@ void fxBeamUpInit (CompScreen *s,
 
 /* burn.c */
 
-Bool
+void
 fxBurnModelStep (CompScreen *s,
 		 CompWindow *w,
 		 float time);
@@ -848,7 +883,7 @@ void fxBurnInit (CompScreen *s,
 
 /* curvedfold.c */
 
-Bool
+void
 fxCurvedFoldModelStep (CompScreen *s,
 		       CompWindow *w,
 		       float time);
@@ -869,11 +904,15 @@ fxDodgeUpdateWindowTransform (CompScreen *s,
 			      CompWindow *w,
 			      CompTransform *wTransform);
 
-Bool
+void
 fxDodgeAnimStep (CompScreen *s,
 		 CompWindow *w,
 		 float time);
 
+void
+fxDodgeUpdateBB (CompOutput *output,
+		 CompWindow * w);
+
 /* domino.c */
 
 void
@@ -883,7 +922,11 @@ fxDomino3DInit (CompScreen *s,
 
 /* dream.c */
 
-Bool
+void
+fxDreamAnimInit (CompScreen * s,
+		 CompWindow * w);
+
+void
 fxDreamModelStep (CompScreen * s,
 		  CompWindow * w,
 		  float time);
@@ -911,11 +954,6 @@ fxFadeUpdateWindowAttrib (AnimScreen *as,
 
 /* focusfade.c */
 
-void
-fxFocusFadeUpdateWindowAttrib2 (AnimScreen *as,
-				CompWindow * w,
-				WindowPaintAttrib *wAttrib);
-
 void
 fxFocusFadeUpdateWindowAttrib (AnimScreen *as,
 			       CompWindow * w,
@@ -943,7 +981,7 @@ fxGlideUpdateWindowAttrib (AnimScreen *as,
 			   CompWindow * w,
 			   WindowPaintAttrib *wAttrib);
 
-Bool
+void
 fxGlideAnimStep (CompScreen *s,
 		 CompWindow *w,
 		 float time);
@@ -968,9 +1006,13 @@ fxGlidePrePaintWindow(CompScreen * s, CompWindow * w);
 void
 fxGlidePostPaintWindow(CompScreen * s, CompWindow * w);
 
+void
+fxGlideUpdateBB (CompOutput *output,
+		 CompWindow *w);
+
 /* horizontalfold.c */
 
-Bool
+void
 fxHorizontalFoldsModelStep (CompScreen *s,
 			    CompWindow *w,
 			    float time);
@@ -1007,7 +1049,7 @@ void
 fxMagicLampInit (CompScreen * s,
 		 CompWindow * w);
 
-Bool
+void
 fxMagicLampModelStep (CompScreen * s,
 		      CompWindow * w,
 		      float time);
@@ -1074,6 +1116,9 @@ void
 drawParticleSystems (CompScreen *s,
 		     CompWindow *w);
 
+void
+particlesUpdateBB (CompOutput *output,
+		   CompWindow * w);
 
 /* polygon.c */
 
@@ -1121,9 +1166,13 @@ void
 polygonsDeceleratingAnimStepPolygon(CompWindow * w,
 				    PolygonObject * p, float forwardProgress);
 
+void
+polygonsUpdateBB (CompOutput *output,
+		  CompWindow * w);
+
 /* rollup.c */
  
-Bool
+void
 fxRollUpModelStep (CompScreen *s,
 		   CompWindow *w,
 		   float time);
@@ -1147,7 +1196,7 @@ fxSkewerAnimStepPolygon (CompWindow * w,
 
 /* wave.c */
  
-Bool
+void
 fxWaveModelStep (CompScreen * s,
 		 CompWindow * w,
 		 float time);
@@ -1179,3 +1228,7 @@ fxSidekickInit (CompScreen *s,
 void
 fxZoomInit (CompScreen * s,
 	    CompWindow * w);
+
+void
+applyZoomTransform (CompWindow * w,
+		    CompTransform *transform);
