@@ -56,13 +56,12 @@ void fxMagicLampInit(CompScreen * s, CompWindow * w)
     ANIM_SCREEN(s);
     ANIM_WINDOW(w);
 
-    Model *model = aw->model;
-
     int screenHeight = s->height;
     aw->minimizeToTop = (WIN_Y(w) + WIN_H(w) / 2) >
 	(aw->icon.y + aw->icon.height / 2);
     int maxWaves;
     float waveAmpMin, waveAmpMax;
+    float distance;
 
     if (aw->curAnimEffect == AnimEffectMagicLamp)
     {
@@ -81,57 +80,65 @@ void fxMagicLampInit(CompScreen * s, CompWindow * w)
     if (waveAmpMax < waveAmpMin)
 	waveAmpMax = waveAmpMin;
 
-    if (maxWaves > 0)
+    if (maxWaves == 0)
     {
-	float distance;
+	aw->magicLampWaveCount = 0;
+	return;
+    }
 
-	if (aw->minimizeToTop)
-	    distance = WIN_Y(w) + WIN_H(w) - aw->icon.y;
-	else
-	    distance = aw->icon.y - WIN_Y(w);
+    // Initialize waves
 
-	// Initialize waves
-	model->magicLampWaveCount =
-	    1 + (float)maxWaves *distance / screenHeight;
+    if (aw->minimizeToTop)
+	distance = WIN_Y(w) + WIN_H(w) - aw->icon.y;
+    else
+	distance = aw->icon.y - WIN_Y(w);
 
-	if (!(model->magicLampWaves))
-	    model->magicLampWaves =
-		calloc(model->magicLampWaveCount, sizeof(WaveParam));
+    aw->magicLampWaveCount =
+	1 + (float)maxWaves *distance / screenHeight;
 
-	int ampDirection = (RAND_FLOAT() < 0.5 ? 1 : -1);
-	int i;
-	float minHalfWidth = 0.22f;
-	float maxHalfWidth = 0.38f;
-
-	for (i = 0; i < model->magicLampWaveCount; i++)
+    if (!(aw->magicLampWaves))
+    {
+	aw->magicLampWaves =
+	    calloc(aw->magicLampWaveCount, sizeof(WaveParam));
+	if (!aw->magicLampWaves)
 	{
-	    model->magicLampWaves[i].amp =
-		ampDirection * (waveAmpMax - waveAmpMin) *
-		rand() / RAND_MAX + ampDirection * waveAmpMin;
-	    model->magicLampWaves[i].halfWidth =
-		RAND_FLOAT() * (maxHalfWidth -
-				minHalfWidth) + minHalfWidth;
-
-	    // avoid offset at top and bottom part by added waves
-	    float availPos = 1 - 2 * model->magicLampWaves[i].halfWidth;
-	    float posInAvailSegment = 0;
-
-	    if (i > 0)
-		posInAvailSegment =
-		    (availPos /
-		     model->magicLampWaveCount) * rand() / RAND_MAX;
-
-	    model->magicLampWaves[i].pos =
-		(posInAvailSegment +
-		 i * availPos / model->magicLampWaveCount +
-		 model->magicLampWaves[i].halfWidth);
-
-	    // switch wave direction
-	    ampDirection *= -1;
+	    compLogMessage (w->screen->display, "animation", CompLogLevelError,
+			    "Not enough memory");
+	    return;
 	}
     }
-    else
-	model->magicLampWaveCount = 0;
+    // Compute wave parameters
+
+    int ampDirection = (RAND_FLOAT() < 0.5 ? 1 : -1);
+    int i;
+    float minHalfWidth = 0.22f;
+    float maxHalfWidth = 0.38f;
+
+    for (i = 0; i < aw->magicLampWaveCount; i++)
+    {
+	aw->magicLampWaves[i].amp =
+	    ampDirection * (waveAmpMax - waveAmpMin) *
+	    rand() / RAND_MAX + ampDirection * waveAmpMin;
+	aw->magicLampWaves[i].halfWidth =
+	    RAND_FLOAT() * (maxHalfWidth -
+			    minHalfWidth) + minHalfWidth;
+
+	// avoid offset at top and bottom part by added waves
+	float availPos = 1 - 2 * aw->magicLampWaves[i].halfWidth;
+	float posInAvailSegment = 0;
+
+	if (i > 0)
+	    posInAvailSegment =
+		(availPos / aw->magicLampWaveCount) * rand() / RAND_MAX;
+
+	aw->magicLampWaves[i].pos =
+	    (posInAvailSegment +
+	     i * availPos / aw->magicLampWaveCount +
+	     aw->magicLampWaves[i].halfWidth);
+
+	// switch wave direction
+	ampDirection *= -1;
+    }
 }
 
 static void
@@ -203,30 +210,10 @@ fxMagicLampModelStepObject(CompWindow * w,
 	    (1 - object->gridPosition.y) * origy +
 	    object->gridPosition.y * icony;
 
+    // Compute current y position
     if (forwardProgress < preShapePhaseEnd)
     {
-	float preShapeProgress = forwardProgress / preShapePhaseEnd;
 	float stretchProgress =	forwardProgress / stretchPhaseEnd;
-	float fx = (iconCloseEndY - object->position.y) / 
-	    (iconCloseEndY - winFarEndY);
-	float fy = (sigmoid(fx) - sigmoid(0)) / (sigmoid(1) - sigmoid(0));
-	float targetx = fy * (origx - iconx) + iconx;
-
-	int i;
-	for (i = 0; i < model->magicLampWaveCount; i++)
-	{
-	    float cosfx =
-		(fx - model->magicLampWaves[i].pos) /
-		model->magicLampWaves[i].halfWidth;
-	    if (cosfx < -1 || cosfx > 1)
-		continue;
-	    targetx +=
-		model->magicLampWaves[i].amp * model->scale.x *
-		(cos(cosfx * M_PI) + 1) / 2;
-	}
-	preShapeProgress = 1 - decelerateProgress(1 - preShapeProgress);
-	object->position.x =
-	    (1 - preShapeProgress) * origx + preShapeProgress * targetx;
 	object->position.y =
 	    (1 - stretchProgress) * origy +
 	    stretchProgress * stretchedPos;
@@ -252,27 +239,42 @@ fxMagicLampModelStepObject(CompWindow * w,
 		postStretchProgress *
 		(stretchedPos + (iconCloseEndY - winFarEndY));
 	}
-	float fx =
-	    (iconCloseEndY - object->position.y) / (iconCloseEndY -
-						    winFarEndY);
-	float fy = (sigmoid(fx) - sigmoid(0)) / (sigmoid(1) - sigmoid(0));
-
-	int i;
-
-	object->position.x = fy * (origx - iconx) + iconx;
-	for (i = 0; i < model->magicLampWaveCount; i++)
-	{
-	    float cosfx =
-		(fx -
-		 model->magicLampWaves[i].pos) /
-		model->magicLampWaves[i].halfWidth;
-	    if (cosfx < -1 || cosfx > 1)
-		continue;
-	    object->position.x +=
-		model->magicLampWaves[i].amp * model->scale.x *
-		(cos(cosfx * M_PI) + 1) / 2;
-	}
     }
+
+    // Compute "target shape" x position
+    float fx = ((iconCloseEndY - object->position.y) / 
+		(iconCloseEndY - winFarEndY));
+    float fy = ((sigmoid(fx) - sigmoid(0)) /
+		(sigmoid(1) - sigmoid(0)));
+    float targetx = fy * (origx - iconx) + iconx;
+
+    // Apply waves
+    int i;
+    for (i = 0; i < aw->magicLampWaveCount; i++)
+    {
+	float cosfx = ((fx - aw->magicLampWaves[i].pos) /
+		       aw->magicLampWaves[i].halfWidth);
+	if (cosfx < -1 || cosfx > 1)
+	    continue;
+	targetx +=
+	    aw->magicLampWaves[i].amp * model->scale.x *
+	    (cos(cosfx * M_PI) + 1) / 2;
+    }
+
+    // Compute current x position
+    if (forwardProgress < preShapePhaseEnd)
+    {
+	float preShapeProgress = forwardProgress / preShapePhaseEnd;
+
+	// Slow down "shaping" toward the end
+	preShapeProgress = 1 - decelerateProgress(1 - preShapeProgress);
+
+	object->position.x =
+	    (1 - preShapeProgress) * origx + preShapeProgress * targetx;
+    }
+    else	    
+	object->position.x = targetx;
+
     if (aw->minimizeToTop)
     {
 	if (object->position.y < iconFarEndY)
@@ -285,10 +287,10 @@ fxMagicLampModelStepObject(CompWindow * w,
     }
 }
 
-Bool fxMagicLampModelStep(CompScreen * s, CompWindow * w, float time)
+void
+fxMagicLampModelStep (CompScreen *s, CompWindow *w, float time)
 {
-    if (!defaultAnimStep(s, w, time))
-	return FALSE;
+    defaultAnimStep (s, w, time);
 
     ANIM_SCREEN(s);
     ANIM_WINDOW(w);
@@ -307,10 +309,11 @@ Bool fxMagicLampModelStep(CompScreen * s, CompWindow * w, float time)
     }
     float forwardProgress = defaultAnimProgress(aw);
 
+    if (aw->magicLampWaveCount > 0 && !aw->magicLampWaves)
+	return;
+
     int i;
     for (i = 0; i < model->numObjects; i++)
 	fxMagicLampModelStepObject(w, model, &model->objects[i],
 				   forwardProgress);
-
-    return TRUE;
 }
