@@ -25,15 +25,19 @@
 #include <string.h>
 #include <stdio.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 #include "parser.h"
 #include "colorfilter_options.h"
 
 static int displayPrivateIndex;
+static int corePrivateIndex;
+
+typedef struct _ColorFilterCore {
+    ObjectAddProc objectAdd;
+} ColorFilterCore;
 
 typedef struct _ColorFilterDisplay
 {
-    HandleEventProc handleEvent;
     int		    screenPrivateIndex;
 } ColorFilterDisplay;
 
@@ -42,7 +46,6 @@ typedef struct _ColorFilterScreen
     int			    windowPrivateIndex;
 
     DrawWindowTextureProc   drawWindowTexture;
-    DamageWindowRectProc    damageWindowRect;
 
     Bool		    isFiltered;
     int			    currentFilter; /* 0 : cumulative mode
@@ -60,20 +63,23 @@ typedef struct _ColorFilterScreen
 typedef struct _ColorFilterWindow
 {
     Bool    isFiltered;
-    Bool    createEvent;
 } ColorFilterWindow;
 
+#define GET_FILTER_CORE(c) \
+    ((ColorFilterCore *) (c)->base.privates[corePrivateIndex].ptr)
+#define FILTER_CORE(c) \
+    ColorFilterCore *fc = GET_FILTER_CORE (c)
 #define GET_FILTER_DISPLAY(d)					    \
-    ((ColorFilterDisplay *) (d)->privates[displayPrivateIndex].ptr)
+    ((ColorFilterDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 #define FILTER_DISPLAY(d)			    \
     ColorFilterDisplay *cfd = GET_FILTER_DISPLAY (d)
 #define GET_FILTER_SCREEN(s, cfd)					\
-    ((ColorFilterScreen *) (s)->privates[(cfd)->screenPrivateIndex].ptr)
+    ((ColorFilterScreen *) (s)->base.privates[(cfd)->screenPrivateIndex].ptr)
 #define FILTER_SCREEN(s)					\
     ColorFilterScreen *cfs = GET_FILTER_SCREEN (s,		\
 			     GET_FILTER_DISPLAY (s->display))
 #define GET_FILTER_WINDOW(w, cfs)					\
-    ((ColorFilterWindow *) (w)->privates[(cfs)->windowPrivateIndex].ptr)
+    ((ColorFilterWindow *) (w)->base.privates[(cfs)->windowPrivateIndex].ptr)
 #define FILTER_WINDOW(w)						\
     ColorFilterWindow *cfw = GET_FILTER_WINDOW  (w,			\
 			     GET_FILTER_SCREEN  (w->screen,		\
@@ -206,7 +212,7 @@ colorFilterToggle (CompDisplay * d, CompAction * action,
 
     w = findWindowAtDisplay (d, xid);
 
-    if (w)
+    if (w && w->screen->fragmentProgram)
 	colorFilterToggleWindow (w);
 
     return TRUE;
@@ -226,7 +232,7 @@ colorFilterToggleAll (CompDisplay * d, CompAction * action,
 
     s = findScreenAtDisplay(d, xid);
 
-    if (s)
+    if (s && s->fragmentProgram)
 	colorFilterToggleScreen (s);
 
     return TRUE;
@@ -246,7 +252,7 @@ colorFilterSwitch (CompDisplay * d, CompAction * action,
 
     s = findScreenAtDisplay(d, xid);
 
-    if (s)
+    if (s && s->fragmentProgram)
 	colorFilterSwitchFilter (s);
 
     return TRUE;
@@ -317,6 +323,11 @@ loadFilters (CompScreen *s, CompTexture *texture)
     for (i = 0; i < count; i++)
     {
 	name = base_name (filters->value[i].s);
+	if (!strlen (name))
+	{
+	    free (name);
+	    continue;
+	}
 	compLogMessage (s->display, "colorfilter", CompLogLevelInfo,
 			"Loading filter %s (item %s).", name,
 			filters->value[i].s);
@@ -333,6 +344,9 @@ loadFilters (CompScreen *s, CompTexture *texture)
 			"Tried to load %d filter(s), %d succeeded.",
 			count, loaded);
 
+    if (!loaded)
+	cfs->filtersCount = 0;
+
     /* Damage currently filtered windows */
     for (w = s->windows; w; w = w->next)
     {
@@ -368,9 +382,9 @@ colorFilterDrawWindowTexture (CompWindow *w, CompTexture *texture,
      *   o Texture's window is filtered */
     /* Note : if required, filter window contents only and not decorations
      * (use that w->texture->name != texture->name for decorations) */
-    if (w->screen->fragmentProgram && cfs->filtersCount && cfw->isFiltered
-	    && (colorfilterGetFilterDecorations (w->screen)
-		|| (texture->name == w->texture->name)))
+    if (cfs->filtersCount && cfw->isFiltered &&
+	(colorfilterGetFilterDecorations (w->screen) ||
+	 (texture->name == w->texture->name)))
     {
 	FragmentAttrib fa = *attrib;
 	if (cfs->currentFilter == 0) /* Cumulative filters mode */
@@ -405,80 +419,42 @@ colorFilterDrawWindowTexture (CompWindow *w, CompTexture *texture,
 }
 
 /*
- * Enable filtering for newly created windows when first damaged
- */
-static Bool
-colorFilterDamageWindowRect (CompWindow * w, Bool initial, BoxPtr rect)
-{
-    int status;
-
-    FILTER_SCREEN (w->screen);
-    FILTER_WINDOW (w);
-
-    /* The window is initial when it is being mapped */
-    if (initial)
-    {
-	/* If the screen is filtered, filter the new window if required */
-	if (cfs->isFiltered && !cfw->isFiltered)
-	    colorFilterToggleWindow (w);
-    }
-
-    UNWRAP (cfs, w->screen, damageWindowRect);
-    status = (*w->screen->damageWindowRect) (w, initial, rect);
-    WRAP (cfs, w->screen, damageWindowRect, colorFilterDamageWindowRect);
-
-    return status;
-}
-
-/*
- * Filter a window on map if required
+ * Filter windows when they are open if they match the filtering rules
  */
 static void
-colorFilterHandleEvent (CompDisplay *d, XEvent *event)
+colorFilterWindowAdd (CompScreen *s,
+		      CompWindow *w)
 {
-    CompWindow *w;
+    FILTER_SCREEN (s);
 
-    FILTER_DISPLAY (d);
-    /* Only apply when window is mapped.
-     * Using CreateNotify doesn't work. */
-    if (event->type == MapNotify)
-    {
-	w = findWindowAtDisplay (d, event->xmap.window);
-	if (w)
-	{
-	    FILTER_WINDOW (w);
-	    if (cfw->createEvent)
-	    {
-		if (w && matchEval (colorfilterGetFilterMatch (w->screen), w))
-		    colorFilterToggleWindow (w);
-		cfw->createEvent = FALSE;
-	    }    
-	}
-    }
-
-    UNWRAP (cfd, d, handleEvent);
-    (*d->handleEvent) (d, event);
-    WRAP (cfd, d, handleEvent, colorFilterHandleEvent);
+    /* cfw->isFiltered is initialized to FALSE in InitWindow, so we only
+       have to toggle it to TRUE if necessary */
+    if (cfs->isFiltered && matchEval (colorfilterGetFilterMatch (s), w))
+	colorFilterToggleWindow (w);
 }
 
-
 /* Internal stuff ----------------------------------------------------------- */
 
 /*
  * Filtering match settings update callback
  */
 static void
-colorfilterFilterMatchsChanged (CompScreen *s, CompOption *opt,
-				ColorfilterScreenOptions num)
+colorFilterMatchsChanged (CompScreen *s, CompOption *opt,
+			  ColorfilterScreenOptions num)
 {
     CompWindow *w;
+
+    FILTER_SCREEN (s);
+
     /* Re-check every window against new match settings */
     for (w = s->windows; w; w = w->next)
     {
 	FILTER_WINDOW (w);
-	if (matchEval (colorfilterGetFilterMatch (s), w)
-		&& !cfw->isFiltered)
+	if (matchEval (colorfilterGetFilterMatch (s), w) &&
+	    cfs->isFiltered && !cfw->isFiltered)
+	{
 	    colorFilterToggleWindow (w);
+	}
     }
 }
 
@@ -486,16 +462,24 @@ colorfilterFilterMatchsChanged (CompScreen *s, CompOption *opt,
  * Exclude match settings update callback
  */
 static void
-colorfilterExcludeMatchsChanged (CompScreen *s, CompOption *opt,
+colorFilterExcludeMatchsChanged (CompScreen *s, CompOption *opt,
 				 ColorfilterScreenOptions num)
 {
     CompWindow *w;
+
+    FILTER_SCREEN (s);
+
     /* Re-check every window against new match settings */
     for (w = s->windows; w; w = w->next)
     {
+	Bool isExcluded;
+
 	FILTER_WINDOW (w);
-	if (matchEval (colorfilterGetFilterMatch (s), w)
-		&& cfw->isFiltered)
+
+	isExcluded = matchEval (colorfilterGetExcludeMatch (s), w);
+	if (isExcluded && cfw->isFiltered)
+	    colorFilterToggleWindow (w);
+	else if (!isExcluded && cfs->isFiltered && !cfw->isFiltered)
 	    colorFilterToggleWindow (w);
     }
 }
@@ -513,6 +497,76 @@ colorFiltersChanged (CompScreen *s, CompOption *opt,
     cfs->filtersLoaded = FALSE;
 }
 
+/*
+ * Damage decorations after the "Filter Decorations" setting got changed
+ */
+static void
+colorFilterDamageDecorations (CompScreen *s, CompOption *opt,
+			      ColorfilterScreenOptions num)
+{
+    damageScreen (s);
+}
+
+static void
+colorFilterObjectAdd (CompObject *parent,
+		      CompObject *object)
+{
+    static ObjectAddProc dispTab[] = {
+	(ObjectAddProc) 0, /* CoreAdd */
+        (ObjectAddProc) 0, /* DisplayAdd */
+        (ObjectAddProc) 0, /* ScreenAdd */
+        (ObjectAddProc) colorFilterWindowAdd
+    };
+
+    FILTER_CORE (&core);
+
+    UNWRAP (fc, &core, objectAdd);
+    (*core.objectAdd) (parent, object);
+    WRAP (fc, &core, objectAdd, colorFilterObjectAdd);
+
+    DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), (parent, object));
+}
+
+static Bool
+colorFilterInitCore (CompPlugin *p,
+		     CompCore   *c)
+{
+    ColorFilterCore *fc;
+
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+        return FALSE;
+
+    fc = malloc (sizeof (ColorFilterCore));
+    if (!fc)
+        return FALSE;
+
+    displayPrivateIndex = allocateDisplayPrivateIndex ();
+    if (displayPrivateIndex < 0)
+    {
+        free (fc);
+        return FALSE;
+    }
+
+    WRAP (fc, c, objectAdd, colorFilterObjectAdd);
+
+    c->base.privates[corePrivateIndex].ptr = fc;
+
+    return TRUE;
+}
+
+static void
+colorFilterFiniCore (CompPlugin *p,
+		     CompCore   *c)
+{
+    FILTER_CORE (c);
+
+    freeDisplayPrivateIndex (displayPrivateIndex);
+
+    UNWRAP (fc, c, objectAdd);
+
+    free (fc);
+}
+
 static Bool
 colorFilterInitDisplay (CompPlugin * p, CompDisplay * d)
 {
@@ -529,12 +583,11 @@ colorFilterInitDisplay (CompPlugin * p, CompDisplay * d)
 	return FALSE;
     }
 
-    colorfilterSetToggleWindowInitiate (d, colorFilterToggle);
-    colorfilterSetToggleScreenInitiate (d, colorFilterToggleAll);
-    colorfilterSetSwitchFilterInitiate (d, colorFilterSwitch);
+    colorfilterSetToggleWindowKeyInitiate (d, colorFilterToggle);
+    colorfilterSetToggleScreenKeyInitiate (d, colorFilterToggleAll);
+    colorfilterSetSwitchFilterKeyInitiate (d, colorFilterSwitch);
 
-    WRAP (cfd, d, handleEvent, colorFilterHandleEvent);
-    d->privates[displayPrivateIndex].ptr = cfd;
+    d->base.privates[displayPrivateIndex].ptr = cfd;
 
     return TRUE;
 }
@@ -544,7 +597,6 @@ colorFilterFiniDisplay (CompPlugin * p, CompDisplay * d)
 {
     FILTER_DISPLAY (d);
     freeScreenPrivateIndex (d, cfd->screenPrivateIndex);
-    UNWRAP (cfd, d, handleEvent);
     free (cfd);
 }
 
@@ -559,7 +611,7 @@ colorFilterInitScreen (CompPlugin * p, CompScreen * s)
     {
 	compLogMessage (s->display, "colorfilter", CompLogLevelFatal,
 			"Fragment program support missing.");
-	return FALSE;
+	return TRUE;
     }
 
     cfs = malloc (sizeof (ColorFilterScreen));
@@ -580,14 +632,14 @@ colorFilterInitScreen (CompPlugin * p, CompScreen * s)
     cfs->filtersFunctions = NULL;
     cfs->filtersCount = 0;
 
-    colorfilterSetFilterMatchNotify (s, colorfilterFilterMatchsChanged);
-    colorfilterSetExcludeMatchNotify (s, colorfilterExcludeMatchsChanged);
+    colorfilterSetFilterMatchNotify (s, colorFilterMatchsChanged);
+    colorfilterSetExcludeMatchNotify (s, colorFilterExcludeMatchsChanged);
     colorfilterSetFiltersNotify (s, colorFiltersChanged);
+    colorfilterSetFilterDecorationsNotify (s, colorFilterDamageDecorations);
 
     WRAP (cfs, s, drawWindowTexture, colorFilterDrawWindowTexture);
-    WRAP (cfs, s, damageWindowRect, colorFilterDamageWindowRect);
 
-    s->privates[cfd->screenPrivateIndex].ptr = cfs;
+    s->base.privates[cfd->screenPrivateIndex].ptr = cfs;
 
     return TRUE;
 }
@@ -599,7 +651,6 @@ colorFilterFiniScreen (CompPlugin * p, CompScreen * s)
 
     freeWindowPrivateIndex (s, cfs->windowPrivateIndex);
     UNWRAP (cfs, s, drawWindowTexture);
-    UNWRAP (cfs, s, damageWindowRect);
 
     unloadFilters (s);
 
@@ -611,6 +662,9 @@ colorFilterInitWindow (CompPlugin * p, CompWindow * w)
 {
     ColorFilterWindow *cfw;
 
+    if (!w->screen->fragmentProgram)
+	return TRUE;
+
     FILTER_SCREEN (w->screen);
 
     cfw = malloc (sizeof (ColorFilterWindow));
@@ -618,9 +672,8 @@ colorFilterInitWindow (CompPlugin * p, CompWindow * w)
 	return FALSE;
 
     cfw->isFiltered = FALSE;
-    cfw->createEvent = TRUE;
 
-    w->privates[cfs->windowPrivateIndex].ptr = cfw;
+    w->base.privates[cfs->windowPrivateIndex].ptr = cfw;
 
     return TRUE;
 }
@@ -628,15 +681,46 @@ colorFilterInitWindow (CompPlugin * p, CompWindow * w)
 static void
 colorFilterFiniWindow (CompPlugin * p, CompWindow * w)
 {
+    if (!w->screen->fragmentProgram)
+	return;
+
     FILTER_WINDOW (w);
     free (cfw);
 }
 
+static CompBool
+colorFilterInitObject (CompPlugin *p,
+		       CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) colorFilterInitCore,
+	(InitPluginObjectProc) colorFilterInitDisplay,
+	(InitPluginObjectProc) colorFilterInitScreen,
+	(InitPluginObjectProc) colorFilterInitWindow
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+colorFilterFiniObject (CompPlugin *p,
+		       CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) colorFilterFiniCore,
+	(FiniPluginObjectProc) colorFilterFiniDisplay,
+	(FiniPluginObjectProc) colorFilterFiniScreen,
+	(FiniPluginObjectProc) colorFilterFiniWindow
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
 static Bool
 colorFilterInit (CompPlugin * p)
 {
-    displayPrivateIndex = allocateDisplayPrivateIndex ();
-    if (displayPrivateIndex < 0)
+    corePrivateIndex = allocateCorePrivateIndex ();
+    if (corePrivateIndex < 0)
 	return FALSE;
 
     return TRUE;
@@ -645,30 +729,16 @@ colorFilterInit (CompPlugin * p)
 static void
 colorFilterFini (CompPlugin * p)
 {
-    if (displayPrivateIndex >= 0)
-	freeDisplayPrivateIndex (displayPrivateIndex);
-}
-
-static int
-colorFilterGetVersion (CompPlugin * p, int version)
-{
-    return ABIVERSION;
+    freeCorePrivateIndex (corePrivateIndex);
 }
 
 CompPluginVTable colorFilterVTable = {
     "colorfilter",
-    colorFilterGetVersion,
     0,
     colorFilterInit,
     colorFilterFini,
-    colorFilterInitDisplay,
-    colorFilterFiniDisplay,
-    colorFilterInitScreen,
-    colorFilterFiniScreen,
-    colorFilterInitWindow,
-    colorFilterFiniWindow,
-    NULL,
-    NULL,
+    colorFilterInitObject,
+    colorFilterFiniObject,
     NULL,
     NULL
 };
