@@ -42,14 +42,15 @@
 #include <X11/Xatom.h>
 #include <X11/cursorfont.h>
 
-#include <compiz.h>
-#include <text.h>
+#include <compiz-core.h>
+#include <compiz-text.h>
 #include "shift_options.h"
 
 typedef enum {
     ShiftStateNone = 0,
     ShiftStateOut,
     ShiftStateSwitching,
+    ShiftStateFinish,
     ShiftStateIn
 } ShiftState;
 
@@ -85,6 +86,8 @@ typedef struct _ShiftDisplay {
     int		    screenPrivateIndex;
     HandleEventProc handleEvent;
 
+    Bool textAvailable;
+
     KeyCode leftKey;
     KeyCode rightKey;
     KeyCode upKey;
@@ -155,6 +158,8 @@ typedef struct _ShiftScreen {
     float lastTitle;
 
     Bool  paintingAbove;
+
+    Bool  canceled;
 } ShiftScreen;
 
 typedef struct _ShiftWindow {
@@ -171,19 +176,19 @@ typedef struct _ShiftWindow {
 #define PI 3.1415926
 
 #define GET_SHIFT_DISPLAY(d)				      \
-    ((ShiftDisplay *) (d)->privates[displayPrivateIndex].ptr)
+    ((ShiftDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define SHIFT_DISPLAY(d)		     \
     ShiftDisplay *sd = GET_SHIFT_DISPLAY (d)
 
 #define GET_SHIFT_SCREEN(s, sd)					  \
-    ((ShiftScreen *) (s)->privates[(sd)->screenPrivateIndex].ptr)
+    ((ShiftScreen *) (s)->base.privates[(sd)->screenPrivateIndex].ptr)
 
 #define SHIFT_SCREEN(s)							   \
     ShiftScreen *ss = GET_SHIFT_SCREEN (s, GET_SHIFT_DISPLAY (s->display))
 
 #define GET_SHIFT_WINDOW(w, ss)					  \
-    ((ShiftWindow *) (w)->privates[(ss)->windowPrivateIndex].ptr)
+    ((ShiftWindow *) (w)->base.privates[(ss)->windowPrivateIndex].ptr)
 
 #define SHIFT_WINDOW(w)					       \
     ShiftWindow *sw = GET_SHIFT_WINDOW  (w,		       \
@@ -255,9 +260,6 @@ isShiftWin (CompWindow *w)
     if (w->state & CompWindowStateSkipTaskbarMask)
 	return FALSE;
 
-    if (w->state & CompWindowStateShadedMask)
-	return FALSE;
-
     if (!matchEval (ss->currentMatch, w))
 	return FALSE;
 
@@ -286,8 +288,13 @@ shiftRenderWindowTitle (CompScreen *s)
     void           *data;
 
     SHIFT_SCREEN (s);
+    SHIFT_DISPLAY (s->display);
 
     shiftFreeWindowTitle (s);
+
+    if (!sd->textAvailable)
+	return;
+
     if (!shiftGetWindowTitle (s))
 	return;
 
@@ -303,8 +310,8 @@ shiftRenderWindowTitle (CompScreen *s)
 	getCurrentOutputExtents (s, &ox1, &oy1, &ox2, &oy2);
 
     /* 75% of the output device as maximum width */
-    tA.maxwidth = (ox2 - ox1) * 3 / 4;
-    tA.maxheight = 100;
+    tA.maxWidth = (ox2 - ox1) * 3 / 4;
+    tA.maxHeight = 100;
     tA.screen = s;
     tA.size = shiftGetTitleFontSize (s);
     tA.color[0] = shiftGetTitleFontColorRed (s);
@@ -313,10 +320,22 @@ shiftRenderWindowTitle (CompScreen *s)
     tA.color[3] = shiftGetTitleFontColorAlpha (s);
     tA.style = (shiftGetTitleFontBold (s)) ?
 	       TEXT_STYLE_BOLD : TEXT_STYLE_NORMAL;
+    tA.style |= TEXT_STYLE_BACKGROUND;
     tA.family = "Sans";
     tA.ellipsize = TRUE;
+    tA.backgroundHMargin = 15;
+    tA.backgroundVMargin = 15;
+    tA.backgroundColor[0] = shiftGetTitleBackColorRed (s);
+    tA.backgroundColor[1] = shiftGetTitleBackColorGreen (s);
+    tA.backgroundColor[2] = shiftGetTitleBackColorBlue (s);
+    tA.backgroundColor[3] = shiftGetTitleBackColorAlpha (s);
+
+
+    if (ss->type == ShiftTypeAll)
+	tA.renderMode = TextRenderWindowTitleWithViewport;
+    else
+	tA.renderMode = TextRenderWindowTitle;
 
-    tA.renderMode = TextRenderWindowTitle;
     tA.data = (void*)ss->selectedWindow;
 
     initTexture (s, &ss->textTexture);
@@ -401,58 +420,6 @@ shiftDrawWindowTitle (CompScreen *s)
 	glEnable (GL_BLEND);
     glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
 
-    glColor4us (shiftGetTitleBackColorRed (s),
-		shiftGetTitleBackColorGreen (s),
-		shiftGetTitleBackColorBlue (s),
-		shiftGetTitleBackColorAlpha (s));
-
-    glPushMatrix ();
-
-    glTranslatef (x, y - height, 0.0f);
-    glRectf (0.0f, height, width, 0.0f);
-    glRectf (0.0f, 0.0f, width, -border);
-    glRectf (0.0f, height + border, width, height);
-    glRectf (-border, height, 0.0f, 0.0f);
-    glRectf (width, height, width + border, 0.0f);
-    glTranslatef (-border, -border, 0.0f);
-
-#define CORNER(a,b) \
-    for (k = a; k < b; k++) \
-    {\
-	float rad = k * (PI / 180.0f);\
-	glVertex2f (0.0f, 0.0f);\
-	glVertex2f (cos(rad) * border, sin(rad) * border);\
-	glVertex2f (cos((k-1) * (PI / 180.0f)) * border, \
-		    sin((k-1) * (PI / 180.0f)) * border);\
-    }
-
-    /* Rounded corners */
-    int k;
-
-    glTranslatef (border, border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (180, 270) glEnd ();
-    glTranslatef (-border, -border, 0.0f);
-
-    glTranslatef (width + border, border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (270, 360) glEnd ();
-    glTranslatef (-(width + border), -border, 0.0f);
-
-    glTranslatef (border, height + border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (90, 180) glEnd ();
-    glTranslatef (-border, -(height + border), 0.0f);
-
-    glTranslatef (width + border, height + border, 0.0f);
-    glBegin (GL_TRIANGLES);
-    CORNER (0, 90) glEnd ();
-    glTranslatef (-(width + border), -(height + border), 0.0f);
-
-    glPopMatrix ();
-
-#undef CORNER
-
     glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
     glColor4f (1.0, 1.0, 1.0, 1.0);
 
@@ -509,7 +476,7 @@ shiftPaintWindow (CompWindow		 *w,
 	if (sw->active)
 	    scaled = (ss->activeSlot != NULL);
 	
-	if (sw->opacity > 0.0 && (ss->activeSlot == NULL))
+	if (sw->opacity > 0.01 && (ss->activeSlot == NULL))
 	{
 	    sAttrib.brightness = sAttrib.brightness * sw->brightness;
 	    sAttrib.opacity = sAttrib.opacity * sw->opacity;
@@ -535,17 +502,24 @@ shiftPaintWindow (CompWindow		 *w,
 	    float sx     = ss->anim * slot->tx;
 	    float sy     = ss->anim * slot->ty;
 	    float sz     = ss->anim * slot->z;
-	    float sscale = (ss->anim * slot->scale) + (1 - ss->anim);
 	    float srot   = (ss->anim * slot->rotation);
+	    float anim   = MIN (1.0, MAX (0.0, ss->anim));
 
+	    float sscale;
 	    float sopacity;
 	    
+
+	    if (slot->primary)
+		sscale = (ss->anim * slot->scale) + (1 - ss->anim);
+	    else
+		sscale = ss->anim * slot->scale;
+	
 	    if (slot->primary && !ss->reflectActive)
 		sopacity = (ss->anim * slot->opacity) + (1 - ss->anim);
 	    else
-		sopacity = ss->anim * slot->opacity;
+		sopacity = anim * anim * slot->opacity;
 
-	    if (sopacity <= 0.0)
+	    if (sopacity <= 0.05)
 		return status;
 
 	    if (mask & PAINT_WINDOW_OCCLUSION_DETECTION_MASK)
@@ -563,8 +537,8 @@ shiftPaintWindow (CompWindow		 *w,
 	    matrixTranslate (&wTransform, sx, sy, sz);
 
 	    matrixTranslate (&wTransform,
-			     w->attrib.x + (w->width  * sscale / 2),
-			     w->attrib.y + (w->height  * sscale / 2.0),
+			     w->attrib.x + (w->attrib.width  * sscale / 2),
+			     w->attrib.y + (w->attrib.height  * sscale / 2.0),
 			     0.0f);
 
 	    matrixScale (&wTransform, ss->output->width, -ss->output->height,
@@ -577,7 +551,7 @@ shiftPaintWindow (CompWindow		 *w,
 
 	    matrixScale (&wTransform, sscale, sscale, 1.0f);
 	    matrixTranslate (&wTransform, -w->attrib.x - (w->width / 2),
-			     -w->attrib.y - (w->height / 2), 0.0f);
+			     -w->attrib.y - (w->attrib.height / 2), 0.0f);
 
 	    glPushMatrix ();
 	    glLoadMatrixf (wTransform.m);
@@ -588,8 +562,7 @@ shiftPaintWindow (CompWindow		 *w,
 	    glPopMatrix ();
 	}
 
-	if (scaled && (ss->state != ShiftStateIn) &&
-	    ((shiftGetOverlayIcon (s) != OverlayIconNone) ||
+	if (scaled && ((shiftGetOverlayIcon (s) != OverlayIconNone) ||
 	     !w->texture->pixmap))
 	{
 	    CompIcon *icon;
@@ -611,12 +584,18 @@ shiftPaintWindow (CompWindow		 *w,
 		float sx       = ss->anim * slot->tx;
 		float sy       = ss->anim * slot->ty;
 		float sz       = ss->anim * slot->z;
-		float sscale   = (ss->anim * slot->scale) + (1 - ss->anim);
 		float srot     = (ss->anim * slot->rotation);
 		float sopacity = ss->anim * slot->opacity;
 
-		scaledWinWidth  = w->width  * sscale;
-		scaledWinHeight = w->height * sscale;
+		float sscale;
+
+		if (slot->primary)
+		    sscale = (ss->anim * slot->scale) + (1 - ss->anim);
+		else
+		    sscale = ss->anim * ss->anim * slot->scale;
+
+		scaledWinWidth  = w->attrib.width  * sscale;
+		scaledWinHeight = w->attrib.height * sscale;
 
 		if (!w->texture->pixmap)
 		    iconOverlay = OverlayIconBig;
@@ -697,7 +676,7 @@ shiftPaintWindow (CompWindow		 *w,
 		    matrixTranslate (&wTransform, w->attrib.x +
 				     (w->width  * sscale / 2),
 				     w->attrib.y +
-				     (w->height  * sscale / 2.0), 0.0f);
+				     (w->attrib.height  * sscale / 2.0), 0.0f);
 	
 		    matrixScale (&wTransform, ss->output->width,
                 		 -ss->output->height, 1.0f);
@@ -708,8 +687,8 @@ shiftPaintWindow (CompWindow		 *w,
                 		 -1.0f / ss->output->height, 1.0f);
 
 		    matrixTranslate (&wTransform, x -
-				     (w->width  * sscale / 2), y -
-				     (w->height  * sscale / 2.0), 0.0f);
+				     (w->attrib.width * sscale / 2), y -
+				     (w->attrib.height * sscale / 2.0), 0.0f);
 		    matrixScale (&wTransform, scale, scale, 1.0f);
 
 		    glPushMatrix ();
@@ -731,7 +710,10 @@ shiftPaintWindow (CompWindow		 *w,
 	
 	if (ss->paintingAbove)
 	{
-	    	sAttrib.opacity = sAttrib.opacity * (1.0 - ss->anim);
+	    sAttrib.opacity = sAttrib.opacity * (1.0 - ss->anim);
+	    
+	    if (ss->anim > 0.99)
+		mask |= PAINT_WINDOW_NO_CORE_INSTANCE_MASK;
 	}
 	
 	UNWRAP (ss, s, paintWindow);
@@ -772,31 +754,23 @@ compareWindows (const void *elem1,
 	w = w->next;
     }
     return -1;
-
-    /*
-    if (w1->mapNum && !w2->mapNum)
-	return -1;
-
-    if (w2->mapNum && !w1->mapNum)
-	return 1;
-
-    return w2->activeNum - w1->activeNum;
-    */
 }
 
-static int 
+static int
 compareShiftWindowDistance (const void *elem1,
 			    const void *elem2)
 {
     float a1   = ((ShiftDrawSlot *) elem1)->distance;
     float a2   = ((ShiftDrawSlot *) elem2)->distance;
+    float ab   = fabs (a1 - a2); 
 
-    if (a1 > a2)
+    if (ab > 0.3 && a1 > a2)
 	return -1;
-    else if (a1 < a2)
+    else if (ab > 0.3 && a1 < a2)
 	return 1;
     else
-	return 0;
+	return compareWindows (&((ShiftDrawSlot *) elem2)->w,
+			       &((ShiftDrawSlot *) elem1)->w);
 }
 
 static Bool
@@ -839,8 +813,8 @@ layoutThumbsCover (CompScreen *s)
 	w = ss->windows[index];
 	SHIFT_WINDOW (w);
 
-	ww = w->width  + w->input.left + w->input.right;
-	wh = w->height + w->input.top  + w->input.bottom;
+	ww = w->attrib.width  + w->input.left + w->input.right;
+	wh = w->attrib.height + w->input.top  + w->input.bottom;
 
 	if (ww > maxThumbWidth)
 	    xScale = (float)(maxThumbWidth) / (float)ww;
@@ -886,7 +860,7 @@ layoutThumbsCover (CompScreen *s)
 		sw->slots[i].scale   = MIN (xScale, yScale);
 		
 		sw->slots[i].y = centerY + (maxThumbHeight / 2.0) -
-				 (((w->height / 2.0) + w->input.bottom) *
+				 (((w->attrib.height / 2.0) + w->input.bottom) *
 				 sw->slots[i].scale);
 
 		if (fabs(distance) < 1.0)
@@ -988,8 +962,8 @@ layoutThumbsFlip (CompScreen *s)
 	w = ss->windows[index];
 	SHIFT_WINDOW (w);
 
-	ww = w->width  + w->input.left + w->input.right;
-	wh = w->height + w->input.top  + w->input.bottom;
+	ww = w->attrib.width  + w->input.left + w->input.right;
+	wh = w->attrib.height + w->input.top  + w->input.bottom;
 
 	if (ww > maxThumbWidth)
 	    xScale = (float)(maxThumbWidth) / (float)ww;
@@ -1035,7 +1009,7 @@ layoutThumbsFlip (CompScreen *s)
 		sw->slots[i].scale   = MIN (xScale, yScale);
 		
 		sw->slots[i].y = centerY + (maxThumbHeight / 2.0) -
-				 (((w->height / 2.0) + w->input.bottom) *
+				 (((w->attrib.height / 2.0) + w->input.bottom) *
 				 sw->slots[i].scale);
 
 		sw->slots[i].x  = sin(angle) * distance * (maxThumbWidth / 2);
@@ -1123,9 +1097,11 @@ shiftAddWindowToList (CompScreen *s,
 static Bool
 shiftUpdateWindowList (CompScreen *s)
 {
-    int i;
     SHIFT_SCREEN (s);
 
+    int        i, idx;
+    CompWindow **wins;
+
     qsort (ss->windows, ss->nWindows, sizeof (CompWindow *), compareWindows);
 
     ss->mvTarget = 0;
@@ -1138,6 +1114,28 @@ shiftUpdateWindowList (CompScreen *s)
 
 	ss->mvTarget++;
     }
+    if (ss->mvTarget == ss->nWindows)
+	ss->mvTarget = 0;
+
+    /* create spetial window order to create a good animation
+       A,B,C,D,E --> A,B,D,E,C to get B,D,E,C,(A),B,D,E,C as initial state */
+    if (shiftGetMode (s) == ModeCover)
+    {
+	wins = malloc(ss->nWindows * sizeof (CompWindow *));
+	if (!wins)
+	    return FALSE;
+	
+	memcpy(wins, ss->windows, ss->nWindows * sizeof (CompWindow *));
+	for (i = 0; i < ss->nWindows; i++)
+	{
+	    idx = ceil (i * 0.5);
+	    idx *= (i & 1) ? 1 : -1;
+	    if (idx < 0)
+		idx += ss->nWindows;
+	    ss->windows[idx] = wins[i];
+	}
+	free (wins);
+    }
 
     return layoutThumbs (s);
 }
@@ -1291,7 +1289,8 @@ adjustShiftWindowAttribs (CompWindow *w, float chunk)
     if ((sw->active && ss->state != ShiftStateIn &&
 	ss->state != ShiftStateNone) ||
 	(shiftGetHideAll(w->screen) && !(w->type & CompWindowTypeDesktopMask) &&
-	(ss->state == ShiftStateOut || ss->state == ShiftStateSwitching)))
+	(ss->state == ShiftStateOut || ss->state == ShiftStateSwitching ||
+	 ss->state == ShiftStateFinish)))
 	opacity = 0.0;
     else
 	opacity = 1.0;
@@ -1400,12 +1399,13 @@ shiftPaintOutput (CompScreen		  *s,
     if (ss->state != ShiftStateNone &&
 	(output->id == ss->usedOutput || output->id == ~0))
     {
-	int           i;
-	
 	CompWindow    *w;
 	CompTransform sTransform = *transform;
-	int oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
-	int oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+	int           i;
+	int           oy1 = s->outputDev[ss->usedOutput].region.extents.y1;
+	int           oy2 = s->outputDev[ss->usedOutput].region.extents.y2;
+	int           maxThumbHeight = (oy2 - oy1) * shiftGetSize(s) / 100;
+	int           oldFilter = s->display->textureFilter;
 
 	if (shiftGetMultioutputMode (s) == MultioutputModeOneBigSwitcher)
 	{
@@ -1413,32 +1413,28 @@ shiftPaintOutput (CompScreen		  *s,
 	    oy2 = s->height;
 	}
 	
-	int maxThumbHeight = (oy2 - oy1) * shiftGetSize(s) / 100;
-
-	int oldFilter = s->display->textureFilter;
-
 	transformToScreenSpace (s, output, -DEFAULT_Z_CAMERA, &sTransform);
 
 	GLdouble clip[4] = { 0.0, -1.0, 0.0, 0.0};
 
-	clip[3] = ((oy1 + (oy2 - oy1)) / 2) + (maxThumbHeight / 2.0);
+	clip[3] = ((oy1 + (oy2 - oy1) / 2)) + (maxThumbHeight / 2.0);
 
 	if (shiftGetReflection (s))
 	{
-	    CompTransform rTransform = sTransform;
-
+	    CompTransform  rTransform = sTransform;
 	    unsigned short color[4];
+	    int            cull, cullInv;
+	    glGetIntegerv (GL_CULL_FACE_MODE, &cull);
+	    cullInv = (cull == GL_BACK)? GL_FRONT : GL_BACK;
 
-	    int maxThumbHeight = (oy2 - oy1) * shiftGetSize(s) / 100;
-
-	    matrixTranslate (&rTransform, 0.0, (oy2 - oy1) + maxThumbHeight,
+	    matrixTranslate (&rTransform, 0.0, oy1 + oy2 + maxThumbHeight,
 			     0.0);
 	    matrixScale (&rTransform, 1.0, -1.0, 1.0);
 
 	    glPushMatrix ();
 	    glLoadMatrixf (rTransform.m);
 
-	    glDisable (GL_CULL_FACE);
+	    glCullFace (cullInv);
 
 	    if (shiftGetMipmaps (s))
 		s->display->textureFilter = GL_LINEAR_MIPMAP_LINEAR;
@@ -1464,7 +1460,7 @@ shiftPaintOutput (CompScreen		  *s,
 	    }
 
 	    glDisable (GL_CLIP_PLANE0);
-	    glEnable( GL_CULL_FACE);
+	    glCullFace (cull);
 
 	    glLoadIdentity();
 	    glTranslatef (0.0, 0.0, -DEFAULT_Z_CAMERA);
@@ -1572,8 +1568,6 @@ shiftPaintOutput (CompScreen		  *s,
 	}	
 	
 	glPopMatrix ();
-
-
     }
 
     return status;
@@ -1664,46 +1658,6 @@ shiftPreparePaintScreen (CompScreen *s,
     WRAP (ss, s, preparePaintScreen, shiftPreparePaintScreen);
 }
 
-static void
-shiftDonePaintScreen (CompScreen *s)
-{
-    SHIFT_SCREEN (s);
-    CompWindow *w;
-
-    if (ss->state != ShiftStateNone)
-    {
-	if (ss->moreAdjust)
-	{
-	    damageScreen (s);
-	}
-	else
-	{
-	    if (ss->moveAdjust)
-	    {
-		damageScreen (s);
-	    }
-
-	    if (ss->state == ShiftStateIn)
-	    {
-		ss->state = ShiftStateNone;
-		shiftActivateEvent(s, FALSE);
-		for (w = s->windows; w; w = w->next)
-		{
-		    SHIFT_WINDOW (w);
-		    sw->active = FALSE;
-		}
-		damageScreen (s);
-	    }
-	    else if (ss->state == ShiftStateOut)
-		ss->state = ShiftStateSwitching;
-	}
-    }
-
-    UNWRAP (ss, s, donePaintScreen);
-    (*s->donePaintScreen) (s);
-    WRAP (ss, s, donePaintScreen, shiftDonePaintScreen);
-}
-
 static Bool
 canStackRelativeTo (CompWindow *w)
 {
@@ -1719,6 +1673,75 @@ canStackRelativeTo (CompWindow *w)
     return TRUE;
 }
 
+static void
+shiftDonePaintScreen (CompScreen *s)
+{
+    SHIFT_SCREEN (s);
+    CompWindow *w;
+
+    if (ss->state != ShiftStateNone)
+    {
+	if (ss->moreAdjust)
+	{
+	    damageScreen (s);
+	}
+	else
+	{
+	    if (ss->state == ShiftStateIn)
+	    {
+		ss->state = ShiftStateNone;
+		shiftActivateEvent(s, FALSE);
+		for (w = s->windows; w; w = w->next)
+		{
+		    SHIFT_WINDOW (w);
+		    sw->active = FALSE;
+		}
+		damageScreen (s);
+	    }
+	    else if (ss->state == ShiftStateOut)
+		ss->state = ShiftStateSwitching;
+
+	    if (ss->moveAdjust)
+	    {
+		damageScreen (s);
+	    }
+	    else if (ss->state == ShiftStateFinish)
+	    {
+		CompWindow *w;
+
+		CompWindow *pw = NULL;
+		int i;
+		
+		ss->state = ShiftStateIn;
+		ss->moreAdjust = TRUE;
+		damageScreen (s);
+
+		if (!ss->canceled && ss->mvTarget != 0)
+		for (i = 0; i < ss->nSlots; i++)
+		{
+		    w = ss->drawSlots[i].w;
+		    if (ss->drawSlots[i].slot->primary && canStackRelativeTo (w))
+		    {
+			if (pw)
+			    restackWindowAbove (w,pw);
+			pw = w;
+		    }
+		}
+
+		if (!ss->canceled && ss->selectedWindow)
+		{
+		    w = findWindowAtScreen (s, ss->selectedWindow);
+		    if (w)
+			sendWindowActivationRequest (s, w->id);
+		}
+	    }
+	}
+    }
+
+    UNWRAP (ss, s, donePaintScreen);
+    (*s->donePaintScreen) (s);
+    WRAP (ss, s, donePaintScreen, shiftDonePaintScreen);
+}
 
 static void
 shiftTerm (CompScreen *s, Bool cancel)
@@ -1733,33 +1756,20 @@ shiftTerm (CompScreen *s, Bool cancel)
 
     if (ss->state != ShiftStateNone)
     {
-	CompWindow *w;
 
-	CompWindow *pw = NULL;
-	int i;
-	
-	for (i = 0; i < ss->nSlots; i++)
+	if (cancel && ss->mvTarget != 0)
 	{
-	    w = ss->drawSlots[i].w;
-	    if (ss->drawSlots[i].slot->primary && canStackRelativeTo (w))
-	    {
-		if (pw)
-		    restackWindowAbove (w,pw);
-		pw = w;
-	    }
+	    if (ss->nWindows - ss->mvTarget > ss->mvTarget)
+		ss->mvAdjust = -ss->mvTarget;
+	    else
+		ss->mvAdjust = ss->nWindows - ss->mvTarget;
+	    ss->moveAdjust = TRUE;
 	}
 
 	ss->moreAdjust = TRUE;
-	ss->state = ShiftStateIn;
+	ss->state = ShiftStateFinish;
+	ss->canceled = cancel;
 	damageScreen (s);
-
-	if (!cancel && ss->selectedWindow)
-	{
-	    w = findWindowAtScreen (s, ss->selectedWindow);
-	    if (w)
-		sendWindowActivationRequest (s, w->id);
-	}
-	
     }
 }
 
@@ -1942,7 +1952,8 @@ shiftInitiate (CompDisplay     *d,
 
 	ss->type = ShiftTypeNormal;
 	
-	if ((ss->state == ShiftStateNone) || (ss->state == ShiftStateIn))
+	if ((ss->state == ShiftStateNone) || (ss->state == ShiftStateIn) ||
+	    (ss->state == ShiftStateFinish))
 	    ret = shiftInitiateScreen (s, action, state, option, nOption);
 	else
 	    ret = shiftTerminate (d, action, state, option, nOption);
@@ -1977,7 +1988,8 @@ shiftInitiateAll (CompDisplay     *d,
 
 	ss->type = ShiftTypeAll;
 	
-	if ((ss->state == ShiftStateNone) || (ss->state == ShiftStateIn))
+	if ((ss->state == ShiftStateNone) || (ss->state == ShiftStateIn) ||
+	    (ss->state == ShiftStateFinish))
 	    ret = shiftInitiateScreen (s, action, state, option, nOption);
 	else
 	    ret = shiftTerminate (d, action, state, option, nOption);
@@ -2211,8 +2223,6 @@ shiftHandleEvent (CompDisplay *d,
 		    ss->startY          = event->xbutton.y_root;
 		    ss->startTarget     = ss->mvTarget + ss->mvAdjust;
 		}
-		else if (event->xbutton.button == Button3)
-		    shiftTerm (s, TRUE);
 	    }
 	}
 	break;
@@ -2228,7 +2238,7 @@ shiftHandleEvent (CompDisplay *d,
 		if (event->xbutton.button == Button1 && ss->buttonPressed)
 		{
 		    int new;
-		    if (event->xbutton.time - ss->buttonPressTime <
+		    if ((int)(event->xbutton.time - ss->buttonPressTime) <
 		        shiftGetClickDuration (s))
 		    	shiftTerm (s, FALSE);
 
@@ -2394,6 +2404,9 @@ shiftInitDisplay (CompPlugin  *p,
 {
     ShiftDisplay *sd;
 
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     sd = malloc (sizeof (ShiftDisplay));
     if (!sd)
 	return FALSE;
@@ -2405,31 +2418,60 @@ shiftInitDisplay (CompPlugin  *p,
 	return FALSE;
     }
 
+    sd->textAvailable = checkPluginABI ("text", TEXT_ABIVERSION);
+    if (!sd->textAvailable)
+	compLogMessage (d, "shift", CompLogLevelWarn,
+			"No compatible text plugin loaded.");
+
     sd->leftKey  = XKeysymToKeycode (d->display, XStringToKeysym ("Left"));
     sd->rightKey = XKeysymToKeycode (d->display, XStringToKeysym ("Right"));
     sd->upKey    = XKeysymToKeycode (d->display, XStringToKeysym ("Up"));
     sd->downKey  = XKeysymToKeycode (d->display, XStringToKeysym ("Down"));
 
-    shiftSetInitiateInitiate (d, shiftInitiate);
-    shiftSetInitiateTerminate (d, shiftTerminate);
-    shiftSetInitiateAllInitiate (d, shiftInitiateAll);
-    shiftSetInitiateAllTerminate (d, shiftTerminate);
-    shiftSetNextInitiate (d, shiftNext);
-    shiftSetNextTerminate (d, shiftTerminate);
-    shiftSetPrevInitiate (d, shiftPrev);
-    shiftSetPrevTerminate (d, shiftTerminate);
-    shiftSetNextAllInitiate (d, shiftNextAll);
-    shiftSetNextAllTerminate (d, shiftTerminate);
-    shiftSetPrevAllInitiate (d, shiftPrevAll);
-    shiftSetPrevAllTerminate (d, shiftTerminate);
-    shiftSetNextGroupInitiate (d, shiftNextGroup);
-    shiftSetNextGroupTerminate (d, shiftTerminate);
-    shiftSetPrevGroupInitiate (d, shiftPrevGroup);
-    shiftSetPrevGroupTerminate (d, shiftTerminate);
+    shiftSetInitiateKeyInitiate (d, shiftInitiate);
+    shiftSetInitiateKeyTerminate (d, shiftTerminate);
+    shiftSetInitiateAllKeyInitiate (d, shiftInitiateAll);
+    shiftSetInitiateAllKeyTerminate (d, shiftTerminate);
+    shiftSetNextKeyInitiate (d, shiftNext);
+    shiftSetNextKeyTerminate (d, shiftTerminate);
+    shiftSetPrevKeyInitiate (d, shiftPrev);
+    shiftSetPrevKeyTerminate (d, shiftTerminate);
+    shiftSetNextAllKeyInitiate (d, shiftNextAll);
+    shiftSetNextAllKeyTerminate (d, shiftTerminate);
+    shiftSetPrevAllKeyInitiate (d, shiftPrevAll);
+    shiftSetPrevAllKeyTerminate (d, shiftTerminate);
+    shiftSetNextGroupKeyInitiate (d, shiftNextGroup);
+    shiftSetNextGroupKeyTerminate (d, shiftTerminate);
+    shiftSetPrevGroupKeyInitiate (d, shiftPrevGroup);
+    shiftSetPrevGroupKeyTerminate (d, shiftTerminate);
+
+    shiftSetInitiateButtonInitiate (d, shiftInitiate);
+    shiftSetInitiateButtonTerminate (d, shiftTerminate);
+    shiftSetInitiateAllButtonInitiate (d, shiftInitiateAll);
+    shiftSetInitiateAllButtonTerminate (d, shiftTerminate);
+    shiftSetNextButtonInitiate (d, shiftNext);
+    shiftSetNextButtonTerminate (d, shiftTerminate);
+    shiftSetPrevButtonInitiate (d, shiftPrev);
+    shiftSetPrevButtonTerminate (d, shiftTerminate);
+    shiftSetNextAllButtonInitiate (d, shiftNextAll);
+    shiftSetNextAllButtonTerminate (d, shiftTerminate);
+    shiftSetPrevAllButtonInitiate (d, shiftPrevAll);
+    shiftSetPrevAllButtonTerminate (d, shiftTerminate);
+    shiftSetNextGroupButtonInitiate (d, shiftNextGroup);
+    shiftSetNextGroupButtonTerminate (d, shiftTerminate);
+    shiftSetPrevGroupButtonInitiate (d, shiftPrevGroup);
+    shiftSetPrevGroupButtonTerminate (d, shiftTerminate);
+
+    shiftSetInitiateEdgeInitiate (d, shiftInitiate);
+    shiftSetInitiateEdgeTerminate (d, shiftTerminate);
+    shiftSetInitiateAllEdgeInitiate (d, shiftInitiateAll);
+    shiftSetInitiateAllEdgeTerminate (d, shiftTerminate);
+
+    shiftSetTerminateButtonInitiate (d, shiftTerminate);
 
     WRAP (sd, d, handleEvent, shiftHandleEvent);
 
-    d->privates[displayPrivateIndex].ptr = sd;
+    d->base.privates[displayPrivateIndex].ptr = sd;
 
     return TRUE;
 }
@@ -2507,7 +2549,7 @@ shiftInitScreen (CompPlugin *p,
 
     ss->cursor = XCreateFontCursor (s->display->display, XC_left_ptr);
 
-    s->privates[sd->screenPrivateIndex].ptr = ss;
+    s->base.privates[sd->screenPrivateIndex].ptr = ss;
 
     return TRUE;
 }
@@ -2556,8 +2598,11 @@ shiftInitWindow (CompPlugin *p,
 
     sw->slots[0].scale = 1.0;
     sw->slots[1].scale = 1.0;
+
+    sw->brightness = 1.0;
+    sw->opacity    = 1.0;
     
-    w->privates[ss->windowPrivateIndex].ptr = sw;
+    w->base.privates[ss->windowPrivateIndex].ptr = sw;
 
     return TRUE;
 }
@@ -2588,27 +2633,41 @@ shiftFini (CompPlugin *p)
 	freeDisplayPrivateIndex (displayPrivateIndex);
 }
 
-static int
-shiftGetVersion (CompPlugin *plugin,
-		int	    version)
+static CompBool
+shiftInitObject (CompPlugin *p,
+		 CompObject *o)
 {
-    return ABIVERSION;
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) shiftInitDisplay,
+	(InitPluginObjectProc) shiftInitScreen,
+	(InitPluginObjectProc) shiftInitWindow
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+shiftFiniObject (CompPlugin *p,
+		 CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) shiftFiniDisplay,
+	(FiniPluginObjectProc) shiftFiniScreen,
+	(FiniPluginObjectProc) shiftFiniWindow
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
 }
 
 CompPluginVTable shiftVTable = {
     "shift",
-    shiftGetVersion,
     0,
     shiftInit,
     shiftFini,
-    shiftInitDisplay,
-    shiftFiniDisplay,
-    shiftInitScreen,
-    shiftFiniScreen,
-    shiftInitWindow,
-    shiftFiniWindow,
-    0,
-    0,
+    shiftInitObject,
+    shiftFiniObject,
     0,
     0
 };
